@@ -10,17 +10,25 @@ mod error;
 mod packet;
 
 use error::*;
-use wd::windows::Windows::{
-    Devices::Custom::{IOControlAccessMode, IOControlBufferingMethod, IOControlCode},
-    Win32::{
-        Foundation::*,
-        Security::*,
-        Storage::FileSystem::*,
-        System::{Diagnostics::Debug::*, Services::*, SystemServices::*, Threading::*},
-    },
-};
 use wd::{address::WINDIVERT_ADDRESS, ioctl::WINDIVERT_IOCTL_RECV};
 use windivert_sys as wd;
+use windows::{
+    core::{Error as WinError, Result as WinResult, HRESULT},
+    Devices::Custom::{IOControlAccessMode, IOControlBufferingMethod, IOControlCode},
+    Win32::{
+        Foundation::{BOOL, ERROR_IO_PENDING, HANDLE, PSTR, WAIT_TIMEOUT},
+        Security::SC_HANDLE,
+        System::{
+            Ioctl::FILE_DEVICE_NETWORK,
+            Services::{
+                CloseServiceHandle, ControlService, OpenSCManagerA, OpenServiceA,
+                SC_MANAGER_ALL_ACCESS, SERVICE_CONTROL_STOP, SERVICE_STATUS,
+            },
+            Threading::{CreateEventA, TlsAlloc, TlsGetValue, TlsSetValue, WAIT_IO_COMPLETION},
+            IO::{CancelIo, DeviceIoControl, GetOverlappedResultEx, OVERLAPPED},
+        },
+    },
+};
 
 pub use error::WinDivertError;
 pub use packet::*;
@@ -35,14 +43,13 @@ use std::{
 };
 
 use etherparse::{InternetSlice, SlicedPacket};
-use windows::{Error as WinError, Result as WinResult, HRESULT};
 
 macro_rules! try_win {
     ($expr:expr) => {{
         let x = $expr;
         if x == BOOL::from(false) {
-            return Err(WinError::fast_error(HRESULT::from_win32(
-                std::io::Error::last_os_error().raw_os_error().unwrap() as u32,
+            return Err(WinError::fast_error(HRESULT(
+                std::io::Error::last_os_error().raw_os_error().unwrap(),
             )));
         } else {
             x
@@ -52,8 +59,8 @@ macro_rules! try_win {
     ($expr:expr, $value:expr) => {{
         let x = $expr;
         if x == $value {
-            return Err(WinError::fast_error(HRESULT::from_win32(
-                std::io::Error::last_os_error().raw_os_error().unwrap() as u32,
+            return Err(WinError::fast_error(HRESULT(
+                std::io::Error::last_os_error().raw_os_error().unwrap(),
             )));
         } else {
             x
@@ -71,16 +78,6 @@ macro_rules! try_divert {
 
 const ADDR_SIZE: usize = std::mem::size_of::<WINDIVERT_ADDRESS>();
 
-const SERVICE_ALL_ACCESS: u32 = STANDARD_RIGHTS_REQUIRED.0
-    | SERVICE_CHANGE_CONFIG
-    | SERVICE_CONTROL_STOP
-    | SERVICE_INTERROGATE
-    | SERVICE_QUERY_CONFIG
-    | SERVICE_QUERY_STATUS
-    | SERVICE_START
-    | SERVICE_STOP
-    | SERVICE_USER_DEFINED_CONTROL;
-
 /// Action parameter for  [`WinDivert::close()`](`fn@WinDivert::close`)
 pub enum CloseAction {
     /// Close the handle and try to uninstall the WinDivert driver.
@@ -123,11 +120,12 @@ impl WinDivert {
     }
 
     fn get_event(tls_idx: u32) -> Result<HANDLE, WinDivertError> {
-        let mut event = HANDLE::NULL;
+        let mut event = HANDLE::default();
         event.0 = unsafe { TlsGetValue(tls_idx) } as isize;
-        if event.is_null() {
-            let event = unsafe { CreateEventA(std::ptr::null_mut(), false, false, PSTR::NULL) };
-            if event.is_null() {
+        if event.is_invalid() {
+            let event =
+                unsafe { CreateEventA(std::ptr::null_mut(), false, false, PSTR::default()) };
+            if event.is_invalid() {
                 return Err(std::io::Error::last_os_error().into());
             } else {
                 unsafe { TlsSetValue(tls_idx, event.0 as *mut c_void) }
@@ -150,7 +148,7 @@ impl WinDivert {
                         let headers = SlicedPacket::from_ip(&buffer)
                             .expect("WinDivert can't capture anything below ip");
                         let offset = match headers.ip.unwrap() {
-                            InternetSlice::Ipv4(ipheader) => ipheader.total_len() as usize,
+                            InternetSlice::Ipv4(ipheader, _) => ipheader.total_len() as usize,
                             InternetSlice::Ipv6(ip6header, _) => {
                                 ip6header.payload_length() as usize + 40
                             }
@@ -286,7 +284,7 @@ impl WinDivert {
         };
 
         if !res.as_bool()
-            && std::io::Error::last_os_error().raw_os_error().unwrap() as u32 == ERROR_IO_PENDING.0
+            && std::io::Error::last_os_error().raw_os_error().unwrap() as u32 == ERROR_IO_PENDING
         {
             loop {
                 let res = unsafe {
@@ -302,7 +300,7 @@ impl WinDivert {
                     break;
                 } else {
                     let return_cause =
-                        (std::io::Error::last_os_error().raw_os_error().unwrap() as u32).into();
+                        std::io::Error::last_os_error().raw_os_error().unwrap() as u32;
                     match return_cause {
                         WAIT_TIMEOUT => {
                             unsafe { CancelIo(self.handle) };
@@ -310,7 +308,7 @@ impl WinDivert {
                         }
                         WAIT_IO_COMPLETION => break,
                         value => {
-                            if let Ok(err) = WinDivertRecvError::try_from(value.0 as i32) {
+                            if let Ok(err) = WinDivertRecvError::try_from(value as i32) {
                                 return Err(WinDivertError::Recv(err));
                             } else {
                                 panic!("This arm should never be reached")
@@ -368,7 +366,7 @@ impl WinDivert {
         };
 
         if !res.as_bool()
-            && std::io::Error::last_os_error().raw_os_error().unwrap() as u32 == ERROR_IO_PENDING.0
+            && std::io::Error::last_os_error().raw_os_error().unwrap() as u32 == ERROR_IO_PENDING
         {
             loop {
                 let res = unsafe {
@@ -385,7 +383,7 @@ impl WinDivert {
                     break;
                 } else {
                     let return_cause =
-                        (std::io::Error::last_os_error().raw_os_error().unwrap() as u32).into();
+                        std::io::Error::last_os_error().raw_os_error().unwrap() as u32;
                     match return_cause {
                         WAIT_TIMEOUT => {
                             unsafe { CancelIo(self.handle) };
@@ -393,7 +391,7 @@ impl WinDivert {
                         }
                         WAIT_IO_COMPLETION => break,
                         value => {
-                            if let Ok(err) = WinDivertRecvError::try_from(value.0 as i32) {
+                            if let Ok(err) = WinDivertRecvError::try_from(value as i32) {
                                 return Err(WinDivertError::Recv(err));
                             } else {
                                 panic!("This arm should never be reached")
@@ -495,12 +493,16 @@ impl WinDivert {
         let status: *mut SERVICE_STATUS = MaybeUninit::uninit().as_mut_ptr();
         unsafe {
             let manager = try_win!(
-                OpenSCManagerA(PSTR::NULL, PSTR::NULL, SERVICE_ALL_ACCESS),
-                SC_HANDLE::NULL
+                OpenSCManagerA(PSTR::default(), PSTR::default(), SC_MANAGER_ALL_ACCESS),
+                SC_HANDLE::default()
             );
             let service = try_win!(
-                OpenServiceA(manager, "WinDIvert", SERVICE_ALL_ACCESS),
-                SC_HANDLE::NULL
+                OpenServiceA(
+                    manager,
+                    PSTR(String::from("WinDivert").as_mut_ptr()),
+                    SC_MANAGER_ALL_ACCESS
+                ),
+                SC_HANDLE::default()
             );
             try_win!(ControlService(service, SERVICE_CONTROL_STOP, status));
             try_win!(CloseServiceHandle(service));
